package org.apidesign.apifest08.currency;

import java.util.*;

final class ConstantRateConverter implements CurrencyConverter {

	private final Currency from;
	private final Currency to;

	private final double rate;

	public ConstantRateConverter(Currency from, Currency to, double rate) throws IllegalArgumentException {
		if (from == null || to == null)
			throw new NullPointerException("None of the currencies can be null");
		if (from.equals(to))
			throw new IllegalArgumentException("Cannot create converter with two equal currencies");
		this.from = from;
		this.to = to;
		this.rate = rate;
	}

	@Override
	public double convert(double value, String from, String to)
			throws CurrencyConversionException, NullPointerException {
		return convert(value, Currency.getInstance(from), Currency.getInstance(to));
	}

	@Override
	public double convert(double value, Currency from, Currency to)
			throws NullPointerException, CurrencyConversionException {

		if (this.from.equals(from)) {
			if (!this.to.equals(to))
				throw new CurrencyConversionException(from, to, "Unsupported currency");
			return value * rate;
		}
		if (this.from.equals(to)) {
			if (!this.to.equals(from))
				throw new CurrencyConversionException(from, to, "Unsupported currency");
			return value / rate;
		}
		throw new CurrencyConversionException(from, to, "Unsupported currency"); 
	}
}
